//THIS IS THE MAIN FILE FOR THE USB TYPEWRITER SOFTWARE
//IT IMPLEMENTS THE SETUP AND MAIN ROUTINES AND CALLS OUT NEEDED LIBRARIES

#define clearbit(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#define setbit(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))

#include <UsbKeyboard.h>
#include <EEPROM.h>
#include "variables.h"
#include <avr/interrupt.h>
#include "WProgram.h"
void setup();
void loop();
void PrintInstructions();
void USBSend(byte usbkeycode);
void USBSend(byte usbkeycode, int mode);
void configPorts();
void SetupTimer2();
void initializeTypewriter();
void initializeUSB();
void delayMs(unsigned int ms);
void delay10us();
void delay1000us();
void listen();
void pulseLow(int pintopulse);
void pulseHigh(int pintopulse);
void recall();
void sendnumber(int number);
void teach();
void GetTeachKey(byte teachkey);
byte waitForKeystroke();
volatile static unsigned int mycounter = 0;

//Setup initializes ports, variables, and so on, then delays 3s.  It also checks if this is teach mode or not.
void setup() {
  delayMs(1); //Delay 1 millisecond
  configPorts();
  digitalWrite(13,HIGH);
  initializeTypewriter(); // Call initialize function.
  initializeUSB();
  SetupTimer2(); // set up timer2 to run every 20 ms (50Hz). timerLoadValue is the value loaded into timer2
  mycounter = 0;
//  TEACH_MODE = true;//debug only!!!
  while (mycounter<200){} //wait for 200*16ms = 3s
}

//Loop is controls whether we are in the Teach, Recall, or Listen thread.
void loop() {
  if(TEACH_MODE){

    teach();
  }
  else if(RECALL_MODE){
    recall();
  }
  else{
 // USBSend(KEY_L);
    listen();
  }
}




//Timer2 overflow interrupt vector handler
//Every time the timer2 overflows, an update command is sent.  This must happen at least once every 50ms.
//This is also a good time to increment a system counter -- mycounter.

ISR(TIMER2_OVF_vect) {
// if (debugLED == true){  
// digitalWrite(13,HIGH);
// debugLED = false;
// }
//else{
//  debugLED = true;
 //   digitalWrite(13,LOW);
//  }
  mycounter = mycounter + 1;

  UsbKeyboard.update(); //this needs to be called every 50ms so the host computer doesn't get bored with us.
  TCNT2 = 1;
}

//This code prints the version number and a brief message when entering teach mode.

void PrintInstructions(){ //CALIBRATING: PLEASE TYPE THE FOLLOWING KEYS

USBSend(KEY_U,UPPER);
USBSend(KEY_S,UPPER);
USBSend(KEY_B,UPPER);
USBSend(KEY_SPACE,UPPER);
USBSend(KEY_T,UPPER);
USBSend(KEY_Y,UPPER);
USBSend(KEY_P,UPPER);
USBSend(KEY_E,UPPER);
USBSend(KEY_W,UPPER);
USBSend(KEY_R,UPPER);
USBSend(KEY_I,UPPER);
USBSend(KEY_T,UPPER);
USBSend(KEY_E,UPPER);
USBSend(KEY_R,UPPER);
USBSend(KEY_9,UPPER);
USBSend(KEY_T,UPPER);
USBSend(KEY_M,UPPER);
USBSend(KEY_0,UPPER);
USBSend(KEY_ENTER);
USBSend(KEY_R,UPPER);
USBSend(KEY_E,UPPER);
USBSend(KEY_V,UPPER);
USBSend(KEY_SPACE,UPPER);
USBSend(KEY_2,LOWER);
USBSend(KEY_PERIOD,LOWER);
USBSend(KEY_4,LOWER);
USBSend(KEY_ENTER);
USBSend(KEY_B,UPPER);
USBSend(KEY_Y,LOWER);
USBSend(KEY_SPACE,UPPER);
USBSend(KEY_J,UPPER);
USBSend(KEY_A,UPPER);
USBSend(KEY_C,UPPER);
USBSend(KEY_K,UPPER);
USBSend(KEY_SPACE,UPPER);
USBSend(KEY_Z,UPPER);
USBSend(KEY_Y,UPPER);
USBSend(KEY_L,UPPER);
USBSend(KEY_K,UPPER);
USBSend(KEY_I,UPPER);
USBSend(KEY_N,UPPER);
USBSend(KEY_ENTER);
USBSend(KEY_U,LOWER);
USBSend(KEY_S,LOWER);
USBSend(KEY_B,LOWER);
USBSend(KEY_T,LOWER);
USBSend(KEY_Y,LOWER);
USBSend(KEY_P,LOWER);
USBSend(KEY_E,LOWER);
USBSend(KEY_W,LOWER);
USBSend(KEY_R,LOWER);
USBSend(KEY_I,LOWER);
USBSend(KEY_T,LOWER);
USBSend(KEY_E,LOWER);
USBSend(KEY_R,LOWER);
USBSend(KEY_PERIOD,LOWER);
USBSend(KEY_C,LOWER);
USBSend(KEY_O,LOWER);
USBSend(KEY_M,LOWER);
USBSend(KEY_ENTER);
USBSend(KEY_C,UPPER);
  USBSend(KEY_A,UPPER);
  USBSend(KEY_L,UPPER);
  USBSend(KEY_I,UPPER);
  USBSend(KEY_B,UPPER);
  USBSend(KEY_R,UPPER);
  USBSend(KEY_A,UPPER);
  USBSend(KEY_T,UPPER);
  USBSend(KEY_I,UPPER);
  USBSend(KEY_N,UPPER);
  USBSend(KEY_G,UPPER);
  USBSend(KEY_PERIOD);
  USBSend(KEY_PERIOD);
  USBSend(KEY_PERIOD);
  USBSend(KEY_T,UPPER);
  USBSend(KEY_Y,UPPER);
  USBSend(KEY_P,UPPER);
  USBSend(KEY_E,UPPER);
  USBSend(KEY_SPACE,UPPER);
  USBSend(KEY_T,UPPER);
  USBSend(KEY_H,UPPER);
  USBSend(KEY_E,UPPER);
  USBSend(KEY_SPACE);
  USBSend(KEY_F,UPPER);
  USBSend(KEY_O,UPPER);
  USBSend(KEY_L,UPPER);
  USBSend(KEY_L,UPPER);
  USBSend(KEY_O,UPPER);
  USBSend(KEY_W,UPPER);
  USBSend(KEY_I,UPPER);
  USBSend(KEY_N,UPPER);
  USBSend(KEY_G,UPPER);
  USBSend(KEY_SPACE);
  USBSend(KEY_K,UPPER);
  USBSend(KEY_E,UPPER);
  USBSend(KEY_Y,UPPER);
  USBSend(KEY_S,UPPER);
  USBSend(KEY_SEMICOLON,UPPER);
  USBSend(KEY_ENTER);
}


//This code sends a keystroke then waits a very short amount of time (to avoid glitching and double strokes)

void USBSend(byte usbkeycode){
    mycounter = 0;
    while(mycounter == 0){} //wait till next interrupt completes
    delay1000us();
    UsbKeyboard.sendKeyStroke(usbkeycode);  //note:usbkeycode is not ascii, it is defined by usb standard
    mycounter = 0;
    while(mycounter <5){} //implement 5*16ms = 80ms delay;
    mycounter = 0;
}

//the function can also be called with a mode, such as LOWER, UPPER, or CTRL
void USBSend(byte usbkeycode, int mode){
    mycounter = 0;
    while(mycounter == 0){}  //wait till next interrupt completes  -- thats so a send command doesn't get interrupted.
    delay1000us();
    switch(mode){
    case LOWER: 
      UsbKeyboard.sendKeyStroke(usbkeycode);
      break;
    case UPPER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_SHIFT_LEFT);
      break;
    case CTRL_LOWER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_CONTROL_LEFT);
      break;
    case CTRL_UPPER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_CONTROL_LEFT|MOD_SHIFT_LEFT);
      break;
    case ALTLOWER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_ALT_LEFT);
      break;
    case ALTUPPER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_ALT_LEFT|MOD_SHIFT_LEFT);
      break;
    case ALTCTRLLOWER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_ALT_LEFT|MOD_CONTROL_LEFT);
      break;
    case ALTCTRLUPPER: 
      UsbKeyboard.sendKeyStroke(usbkeycode,MOD_ALT_LEFT|MOD_CONTROL_LEFT|MOD_SHIFT_LEFT);
      break;
    }

    mycounter = 0;
    while(mycounter <5){} //implement 5*16ms = 80ms delay;
    mycounter = 0;
}


void configPorts() {
  pinMode(SERpin, OUTPUT);
  pinMode(RCLKpin, OUTPUT);
  pinMode(SRCLRpin, OUTPUT);
  pinMode(SPACEBARpin, INPUT);
  pinMode(KEYpin, INPUT);
  pinMode(7, INPUT);
  pinMode(8,INPUT);
  pinMode(9,INPUT);
  pinMode(10,INPUT);
  pinMode(SHIFTpin,INPUT);
  pinMode(ALTpin,INPUT);
  pinMode(CTRLpin,INPUT);
              
  pinMode(TEACHpin, INPUT);
  
  digitalWrite(TEACHpin,HIGH);//activate internal pullup resistor 
  digitalWrite(7,HIGH);
  digitalWrite(8,HIGH);
  digitalWrite(9,HIGH);
  digitalWrite(10,HIGH);
  digitalWrite(SHIFTpin,HIGH);
  digitalWrite(CTRLpin,HIGH);
  digitalWrite(ALTpin,HIGH);
  digitalWrite(S2,HIGH);
  
     asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
     
  polarity7 = digitalRead(7);
  polarity8 = digitalRead(8);
  polarity9 = digitalRead(9);
  polarity10 = digitalRead(10);
  polarityshift = digitalRead(SHIFTpin);
  polarityalt = digitalRead(ALTpin);
  polarityctrl = digitalRead(CTRLpin);

}

//this code sets the configuration bits that initialize the timer.

void SetupTimer2(){   //timeoutFrequency must be less than 60Hz (every 16 ms)
  
   //Timer2 Settings:  Timer Prescaler /1024 = 16kHz or 16ms per overflow (coincidentally).
  TCCR2A = 0;
  TCCR2B = (1<<CS22 | 1<<CS21 | 1<<CS20);

  //Timer2 Overflow Interrupt Enable
 TIMSK2 = 1<<TOIE2; 

  //Timer2 Overflow Interrupt Enable
 // TIMSK |= (1 << TOIE2);
  TCNT2 = 1;


}

void initializeTypewriter() {
  if(digitalRead(TEACHpin) == LOW){  //spacebar controls teachmode 
    TEACH_MODE = true;
    RECALL_MODE = false;
  }
  else{
    TEACH_MODE = false;
    RECALL_MODE = true;
  }
 
 //PUT SHIFT REGISTER INTO A KNOWN STATE (all zeros) AND INITIALIZE KEYTOCHARARRAY TO ALL -1 
  pulseLow(SRCLRpin);  // reset registers
  digitalWrite(SERpin,LOW); //clock 100 zeros into the shift register
  for (int init_i=0; init_i <= 199; init_i++){
     pulseHigh(RCLKpin);
     keytocharARRAY[init_i] = INVALID;// until explicity programmed, each value in the array returns invalid.
  }
    keytocharARRAY[SPACEBARsensor]=KEY_SPACE;

  
}

//This code enumerates the USB to the host.  It also includes routines to make delays of different length to replace the Arduino's built in delay functions (which use interrupts that have been disabled).

void initializeUSB(){
    TIMSK0&=!(1<<TOIE0);  //disable Timer0 interrupts
    cli();   // Clear interrupts while performing time-critical operations
    
    // Force re-enumeration so the host will detect us 
    usbDeviceDisconnect(); 
    delayMs(250); 
    usbDeviceConnect();   
    
    // Set interrupts again  
    sei();
    
}

void delayMs(unsigned int ms){ 
    /** * Define our own delay function so that we don't have to rely on * 
operation of timer0, the interrupt used by the internal delay() */
  for (int ms_i = 0; ms_i < ms; ms_i++){   
    delayMicroseconds(1000); 
    }
} 

void delay10us(){
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
  asm("nop;nop;nop;nop;");
}

void delay1000us(){
  for(int del_i =0; del_i<100; del_i++){
    delay10us();
  }
}


//this code detects a keystroke using waitforkeystroke() and then sends the result using USBSend().  It also checks that the key is valid and isn't a repeat.  It also adds shift, ctrl, and fn functionality.

void listen(){
//LISTEN FOR KEYSTROKE AND SEND IT TO PC 
    byte keypressed = 0;
    byte modekeys = 0;
    
    waitcount = 0;
    keypressed = waitForKeystroke();

    if (digitalRead(FNpin) == 0){
          setbit(modekeys,CTRL_F);
    }

    if (digitalRead(SHIFTpin) != polarityshift){
          setbit(modekeys,SHIFT_F);
    }
 if (digitalRead(ALTpin) != polarityalt){
          if (keytocharARRAY[keypressed+100] != INVALID){keypressed += 100;}
          else{setbit(modekeys,ALT_F);}
   }
    if (digitalRead(S2) == 0){  //S2 is used as a secondary alt pin
          if (keytocharARRAY[keypressed+100] != INVALID){keypressed += 100;}
          else{setbit(modekeys,ALT_F);}
    }
    if (digitalRead(CTRLpin) != polarityctrl){
          setbit(modekeys,CTRL_F);
    }
    
  // if this is not the same letter as before (on the +0 or +100 keymap), 
 //or if an appropriate time has been elapsed between two hits of the same key,
 // and the key is valid
 //send the character over USB (in upper or lower case)
       
    if (((keypressed%100 != lastkeypressed)||(waitcount>25)||(repeatcount>10))&&(keypressed != INVALID)){ 
             USBSend((byte)keytocharARRAY[keypressed],modekeys);

    }
    
    if (keypressed != INVALID){
      lastkeypressed = keypressed%100; // lastkeypressed must be a number between 1 and 256
    }      
 }

//This is a very low level routine to pulse a pine high then low again, or vice versa.

void pulseLow(int pintopulse){
  delay10us();
  digitalWrite(pintopulse,LOW);
  delay10us();
  digitalWrite(pintopulse,HIGH);
  delay10us();
}

void pulseHigh(int pintopulse){
  delay10us();
  digitalWrite(pintopulse,HIGH);
  delay10us();
  digitalWrite(pintopulse,LOW);
  delay10us();
}

//this routine loads key values from the eeprom.

void recall(){
  //GET THE KEY/CHAR PAIRS THAT HAVE BEEN STORED IN EEPROM
  for (byte recall_i =0; recall_i <= 99; recall_i++){
  keytocharARRAY[recall_i] = EEPROM.read(recall_i); 
  }
    RECALL_MODE = false; // end of recall mode

  for (byte recall_i =100; recall_i <= 199; recall_i++){
  keytocharARRAY[recall_i] = EEPROM.read(recall_i); 
  }
    RECALL_MODE = false; // end of recall mode
}


void sendnumber(int number){
    int ones ;
    int tens;
    int hundreds;
    
    ones = number%10;
    tens = ((number - ones)%100)/10;
    hundreds = ((number - ones - 10*tens))/100;
  
    mycounter = 0;
    while(mycounter<2){}// implement  delay
  
     if (hundreds != 0){
      USBSend(KEY_8,UPPER);
    }
    else{
//      USBSend(39,LOWER);
    }
  
    mycounter = 0;
    while(mycounter<2){}// implement delay
     
    if (tens != 0){
      USBSend(29+tens,LOWER);
    }
    else{
      USBSend(39,LOWER);
    }

   mycounter = 0;
   while(mycounter<2){}// implement delay  
 
    if (ones!=0){
      USBSend(29+ones,LOWER);
    }
    else{
      USBSend(39,LOWER);
    }
    
   mycounter = 0;
   while(mycounter<2){}// implement delay 
    
}

//This code launches the calibration mode (only happens when teachpin is low on startup).

void teach(){
  byte keypressed;
  PrintInstructions();
  
//CYCLE THROUGH CHARACTERS:
 // for (char learnChar = KEY_A;  learnChar <= KEY_Z ;  learnChar++){  //key_a and key_z are variables declared in usbkeyboard.h
for (byte learnChar = KEY_A; learnChar <= KEY_Z; learnChar ++){
   USBSend(learnChar);
   USBSend(51,UPPER);  //51 is USB code for ";"  but shift left makes it ":"
   GetTeachKey(learnChar);
   USBSend(KEY_ENTER);
  }  

for (byte learnChar = KEY_1; learnChar <= KEY_0; learnChar ++){
   USBSend(learnChar);
   USBSend(51,UPPER);  //51 is USB code for ";"  but shift left makes it ":"
   GetTeachKey(learnChar);
   USBSend(KEY_ENTER);
  }  
//Learn which key is the spacebar.  (this code is optional)

   USBSend(KEY_PERIOD);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_PERIOD);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_COMMA);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_COMMA);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_SLASH);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_SLASH);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_SEMICOLON);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_SEMICOLON);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_QUOTES);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_QUOTES);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_B);
   USBSend(KEY_A);
   USBSend(KEY_C);
   USBSend(KEY_K);
   USBSend(KEY_S);
   USBSend(KEY_P);
   USBSend(KEY_A);
   USBSend(KEY_C);
   USBSend(KEY_E);
   USBSend(KEY_R);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_BACKSPACE);
   USBSend(KEY_ENTER);
   
      
   USBSend(KEY_D);
   USBSend(KEY_E);
   USBSend(KEY_L);
   USBSend(KEY_E);
   USBSend(KEY_T);
   USBSend(KEY_E);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_DELETE);
   USBSend(KEY_ENTER);

   
   USBSend(KEY_DASH);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_DASH);   
   USBSend(KEY_ENTER);
   
   USBSend(KEY_EQ);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_EQ);   
   USBSend(KEY_ENTER);

   USBSend(KEY_BACKSLASH);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_BACKSLASH);   
   USBSend(KEY_ENTER);

   USBSend(KEY_LBRACKET);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_LBRACKET);   
   USBSend(KEY_ENTER);   
   
   USBSend(KEY_RBRACKET);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_RBRACKET);   
   USBSend(KEY_ENTER);
   
   USBSend(KEY_TILDE,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_TILDE);   
   USBSend(KEY_ENTER);   
   
   USBSend(KEY_L,UPPER);
   USBSend(KEY_E,UPPER);
   USBSend(KEY_F,UPPER);
   USBSend(KEY_T,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_LEFT);   
   USBSend(KEY_ENTER);   
   
   USBSend(KEY_R,UPPER);
   USBSend(KEY_I,UPPER);
   USBSend(KEY_G,UPPER);
   USBSend(KEY_H,UPPER);
   USBSend(KEY_T,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_RIGHT);   
   USBSend(KEY_ENTER);   
   
   USBSend(KEY_U,UPPER);
   USBSend(KEY_P,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_UP);   
   USBSend(KEY_ENTER);   
   
   USBSend(KEY_D,UPPER);
   USBSend(KEY_O,UPPER);
   USBSend(KEY_W,UPPER);
   USBSend(KEY_N,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_DOWN);   
   USBSend(KEY_ENTER);  
   
   USBSend(KEY_T,UPPER);
   USBSend(KEY_A,UPPER);
   USBSend(KEY_B,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_TAB);   
   USBSend(KEY_ENTER);
      
   USBSend(KEY_E,UPPER);
   USBSend(KEY_S,UPPER);
   USBSend(KEY_C,UPPER);
   USBSend(KEY_SEMICOLON,UPPER); 
   GetTeachKey(KEY_ESC);
   USBSend(KEY_ENTER);
   
   for (int f_index = 1; f_index<=12;f_index++){
     USBSend(KEY_F,UPPER);
     sendnumber(f_index);
     USBSend(KEY_SEMICOLON,UPPER);
     GetTeachKey(KEY_F1+f_index-1);
     USBSend(KEY_ENTER);
   }
   
   USBSend(KEY_S);
   USBSend(KEY_P);
   USBSend(KEY_A);
   USBSend(KEY_C);
   USBSend(KEY_E);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_SPACE);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_E);
   USBSend(KEY_N);
   USBSend(KEY_T);
   USBSend(KEY_E);
   USBSend(KEY_R);
   USBSend(KEY_SEMICOLON,UPPER);
   GetTeachKey(KEY_ENTER);
   USBSend(KEY_ENTER);
   
   USBSend(KEY_C);
   USBSend(KEY_A);
   USBSend(KEY_L);
   USBSend(KEY_I);
   USBSend(KEY_B);
   USBSend(KEY_R);
   USBSend(KEY_A);
   USBSend(KEY_T);
   USBSend(KEY_I);
   USBSend(KEY_O);
   USBSend(KEY_N);
   USBSend(KEY_SPACE);
   USBSend(KEY_S);
   USBSend(KEY_A);
   USBSend(KEY_V);
   USBSend(KEY_E);
   USBSend(KEY_D);
   USBSend(KEY_1,UPPER);
   USBSend(KEY_ENTER);
   //end of optional code   
   
   
   
//STORE ARRAY IN EEPROM:
  for (int key_i = 1; key_i <=200; key_i++){ 
    EEPROM.write(key_i,keytocharARRAY[key_i]);  //store key/char pair in eeprom.  key # is the eeprom address,  keytocharARRAY(key_i) is the character stored there.
  }
  
//  for (int key_i = 200; key_i <=214; key_i++){ 
//    EEPROM.write(key_i,keytocharARRAY[key_i]);  //store key/char pair in eeprom.  key # is the eeprom address,  keytocharARRAY(key_i) is the character stored there.
//  }
  TEACH_MODE = false; // exit teach mode
//  Serial.print("done programming");

}

void GetTeachKey(byte teachkey){
   int keypressed = 0;    
   while(keypressed == 0){
   keypressed = waitForKeystroke();
   }
//   keytocharARRAY[keypressed] = teachkey;
       if (digitalRead(S2) == 0){
        keytocharARRAY[keypressed+100] = teachkey;   
        sendnumber(keypressed+100);
      }
      else{
        keytocharARRAY[keypressed] = teachkey;
        sendnumber(keypressed);
      }
  
   mycounter = 0;
   while(mycounter<30){}// implement 500ms delay
   mycounter = 0;
}
  

//This is the meat of the code.   It shifts bits down the shift registers until a signal is received back at the KEYpin, which tells it what key has been pressed.

byte waitForKeystroke(){
  byte key = 0;
  byte doublecheckkey  = 0;
  unsigned int keypress_count = 0;
//LOOP UNTIL A KEY IS PRESSED  
  while(key == 0){
   if(waitcount<30000){waitcount++;}
   digitalWrite(SERpin,HIGH);
   pulseHigh(RCLKpin);
   digitalWrite(SERpin,LOW);
   asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
   asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
    for (int shift_i = 1; shift_i<95;shift_i++){
      pulseHigh(RCLKpin);
      if (digitalRead(KEYpin) == HIGH){
        key = shift_i;
      }
    }
       if(digitalRead(7) != polarity7){key = 96;}  
       if(digitalRead(8) != polarity8){key = 97;}
       if(digitalRead(9) != polarity9){key = 98;}
       if(digitalRead(10) != polarity10){key = 99;}
//       if(digitalRead(TEACHpin) == 0){key = 96;}//teachpin used as general input for modvic

   }
   digitalWrite(SERpin,HIGH);
   pulseHigh(RCLKpin);
   digitalWrite(SERpin,LOW);
   asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
   asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
   for (int shift_i = 1; shift_i<95;shift_i++){
      pulseHigh(RCLKpin);
      if (digitalRead(KEYpin) == HIGH){
        doublecheckkey = shift_i;
      }
   }

   if(doublecheckkey != key){key = INVALID;}
   if(digitalRead(7) != polarity7){key = 96;} // commented out for modvic
   if(digitalRead(8) != polarity8){key = 97;}
   if(digitalRead(9) != polarity9){key = 98;}
   if(digitalRead(10) != polarity10){key = 99;}
//   if(digitalRead(TEACHpin) == 0){key = 96;} //teachpin used as general input for modvic
return key;
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

